Attribute VB_Name = "Win32"
'
' WinAPI registry functions wrappers
'

Option Explicit

Private Const KEY_QUERY_VALUE = &H1
Private Const REG_SZ = 1
Private Const REG_DWORD = 4
Private Const ERROR_SUCCESS = 0
Public Const HKEY_LOCAL_MACHINE = &H80000002
Public Const GWL_WNDPROC = -4
Public Const WM_COPYDATA = &H4A
Public Const SW_SHOW = 1

Private Declare Function RegOpenKeyEx Lib "advapi32" Alias "RegOpenKeyExA" (ByVal HKEY&, ByVal lpSubKey$, ByVal dwReserved&, ByVal samDesired As Long, phkResult As Long) As Long
Private Declare Function RegQueryValueEx Lib "advapi32" Alias "RegQueryValueExA" (ByVal HKEY As Long, ByVal lpValueName As String, lpReserved As Long, lpType As Long, lpData As Any, lpcbData As Long) As Long
Private Declare Function RegCloseKey& Lib "advapi32" (ByVal HKEY&)

Private Declare Function CallWindowProc Lib "user32" Alias "CallWindowProcA" (ByVal lpPrevWndFunc As Long, ByVal hwnd As Long, ByVal Msg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Private Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" (ByVal hwnd As Long, ByVal nIndex As Long, ByVal dwNewLong As Long) As Long
Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (hpvDest As Any, hpvSource As Any, ByVal cbCopy As Long)
Private Declare Function BringToFront Lib "user32" Alias "SetForegroundWindow" (hwnd As Long) As Long

Public Declare Function GetPrivateProfileInt Lib "kernel32" Alias "GetPrivateProfileIntA" (ByVal lpApplicationName As String, ByVal lpKeyName As String, ByVal nDefault As Long, ByVal lpFileName As String) As Long
Public Declare Function GetPrivateProfileString Lib "kernel32" Alias "GetPrivateProfileStringA" (ByVal lpApplicationName As String, ByVal lpKeyName As Any, ByVal lpDefault As String, ByVal lpReturnedString As String, ByVal nSize As Long, ByVal lpFileName As String) As Long
Public Declare Function ShellExecute Lib "shell32.dll" Alias "ShellExecuteA" (ByVal hwnd As Long, ByVal lpOperation As String, ByVal lpFile As String, ByVal lpParameters As String, ByVal lpDirectory As String, ByVal nShowCmd As Long) As Long

Public lpPrevWndProc As Long

Private Type COPYDATASTRUCT
    dwData As Long
    cbData As Long
    lpData As Long
End Type

Public Function GetRegDWord(HKEY As Long, strSubKey As String, strValueName As String, lngDefaultValue As Long) As Long

    Dim lngDataLen As Long
    Dim hSubKey As Long
    Dim lngRetVal As Long

    If RegOpenKeyEx(HKEY, strSubKey, 0, KEY_QUERY_VALUE, hSubKey) = ERROR_SUCCESS Then
        lngDataLen = 4
        If RegQueryValueEx(hSubKey, strValueName, ByVal 0, REG_DWORD, lngRetVal, lngDataLen) = ERROR_SUCCESS Then
            GetRegDWord = lngRetVal
        Else: GetRegDWord = lngDefaultValue
        End If
        RegCloseKey hSubKey
    Else: GetRegDWord = lngDefaultValue
    End If
    
End Function

Public Function GetRegString(HKEY As Long, strSubKey As String, strValueName As String, strDefaultValue As String) As String
    
    Dim strSetting As String
    Dim lngDataLen As Long
    Dim hSubKey As Long
    
    If RegOpenKeyEx(HKEY, strSubKey, 0, KEY_QUERY_VALUE, hSubKey) = ERROR_SUCCESS Then
        strSetting = Space(255)
        lngDataLen = Len(strSetting)
        If RegQueryValueEx(hSubKey, strValueName, ByVal 0, REG_SZ, ByVal strSetting, lngDataLen) = ERROR_SUCCESS Then
            If lngDataLen > 1 Then
                GetRegString = Left(strSetting, lngDataLen - 1)
            End If
        Else: GetRegString = strDefaultValue
        End If
        RegCloseKey hSubKey
    Else: GetRegString = strDefaultValue
    End If
    
End Function

' Listen for operating system messages
Public Function WindowProc(ByVal hw As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long

    Select Case uMsg
        Case WM_COPYDATA ' analyze the WM_COPYDATA message
            
            Dim Data As COPYDATASTRUCT
            
            CopyMemory Data, ByVal lParam, ByVal Len(Data)
            
            ' if message ID is equal to ID from the COPYDATASTRUCT then the message has been received from the virtual printer
            If Data.dwData = SampleForm.lMessageID Then
                
                Dim sData As String
                ' prepare the buffer
                sData = Space(Data.cbData)
                ' copy the data to the string buffer
                CopyMemory ByVal StrPtr(sData), ByVal Data.lpData, ByVal Data.cbData
                ' get the print job info file path from the buffer
                SampleForm.sInputIniFilePath = StrConv(sData, vbUnicode)
                ' fill the listbox
                SampleForm.FillInfo
                ' bring the window to top
                BringToFront (SampleForm.hwnd)
                
            End If
                        
    End Select

    WindowProc = CallWindowProc(lpPrevWndProc, hw, uMsg, wParam, lParam)
    
End Function

Public Sub StartSubclass(hwnd As Long)
    
    lpPrevWndProc = SetWindowLong(hwnd, GWL_WNDPROC, AddressOf WindowProc)
    
End Sub

Public Sub StopSubclass(hwnd As Long)
    
    If lpPrevWndProc <> 0 Then
        SetWindowLong hwnd, GWL_WNDPROC, lpPrevWndProc
        lpPrevWndProc = 0
    End If
    
End Sub

